import React from 'react';
import {
  useAppDispatch,
  useGraphQueryResult,
  useSchemaGraph,
  useQuerybuilderGraph,
  useSearchResult,
} from '@graphpolaris/shared/lib/data-access';
import { filterData } from './similarity';
import {
  addSearchResultData,
  addSearchResultSchema,
  addSearchResultQueryBuilder,
  resetSearchResults,
} from '@graphpolaris/shared/lib/data-access/store/searchResultSlice';
import { Close } from '@mui/icons-material';

const SIMILARITY_THRESHOLD = 0.7;

const CATEGORY_ACTIONS = {
  data: (payload: { nodes: Record<string, any>[]; edges: Record<string, any>[] }, dispatch) => {
    dispatch(addSearchResultData(payload));
  },
  schema: (payload: { nodes: Record<string, any>[]; edges: Record<string, any>[] }, dispatch) => {
    dispatch(addSearchResultSchema(payload));
  },
  querybuilder: (payload: { nodes: Record<string, any>[]; edges: Record<string, any>[] }, dispatch) => {
    dispatch(addSearchResultQueryBuilder(payload));
  },
};

const SEARCH_CATEGORIES: string[] = Object.keys(CATEGORY_ACTIONS);

export default function SearchBar({}) {
  const inputRef = React.useRef<HTMLInputElement>(null);
  const dispatch = useAppDispatch();
  const results = useSearchResult();
  const schema = useSchemaGraph();
  const graphData = useGraphQueryResult();
  const querybuilderData = useQuerybuilderGraph();
  const [search, setSearch] = React.useState<string>('');
  const [inputActive, setInputActive] = React.useState<boolean>(false);

  const dataSources = {
    data: graphData,
    schema: schema,
    querybuilder: querybuilderData,
  };

  React.useEffect(() => {
    handleSearch();
  }, [search]);

  const handleSearch = () => {
    let query = search.toLowerCase();
    const categories = search.match(/@[^ ]+/g);

    if (categories) {
      categories.map((category) => {
        query = query.replace(category, '').trim();
        const cat = category.substring(1);

        if (cat in CATEGORY_ACTIONS) {
          const categoryAction = CATEGORY_ACTIONS[cat];
          const data = dataSources[cat];

          const payload = {
            nodes: filterData(query, data.nodes, SIMILARITY_THRESHOLD),
            edges: filterData(query, data.edges, SIMILARITY_THRESHOLD),
          };

          categoryAction(payload, dispatch);
        }
      });
    } else {
      for (const category of SEARCH_CATEGORIES) {
        const categoryAction = CATEGORY_ACTIONS[category];
        const data = dataSources[category];

        const payload = {
          nodes: filterData(query, data.nodes, SIMILARITY_THRESHOLD),
          edges: filterData(query, data.edges, SIMILARITY_THRESHOLD),
        };

        categoryAction(payload, dispatch);
      }
    }
  };

  React.useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      if (inputRef.current && !inputRef.current.contains(event.target)) {
        setSearch('');
        // dispatch(resetSearchResults());
        inputRef.current.blur();
      }
    };
    document.addEventListener('click', handleClickOutside);
    return () => {
      document.removeEventListener('click', handleClickOutside);
    };
  }, []);

  return (
    <div className={`relative form-control mr-4 ${inputActive || search !== '' ? 'w-96' : 'w-56'} transition-width duration-300`}>
      <div className="relative rounded-md shadow-sm w-full">
        <input
          ref={inputRef}
          value={search}
          onChange={(e) => setSearch(e.target.value)}
          type="text"
          onFocus={() => setInputActive(true)}
          onBlur={() => setInputActive(false)}
          placeholder="Search…"
          className="block w-full rounded-md border-0 py-1.5 pl-2 pr-8 text-gray-900 ring-1 ring-inset ring-gray-300 placeholder:text-gray-400 sm:text-sm sm:leading-6"
        />
        {search !== '' && (
          <div
            className="absolute inset-y-0 right-0 flex items-center cursor-pointer"
            onClick={() => {
              dispatch(resetSearchResults());
              setSearch('');
            }}
          >
            <div className="py-0 px-2">
              <span className="text-gray-400 text-xs">
                <Close />
              </span>
            </div>
          </div>
        )}
      </div>
      {search !== '' && (
        <div className="absolute top-[calc(100%+0.5rem)] z-[1] bg-white rounded-none card-bordered w-full">
          {SEARCH_CATEGORIES.map((category, index) => {
            if (results[category].nodes.length > 0 || results[category].edges.length > 0) {
              return (
                <div key={index}>
                  <div className="flex justify-between p-2">
                    <p className="text-entity-500">{category.charAt(0).toUpperCase() + category.slice(1)}</p>
                    <p className="font-thin text-slate-400">{results[category].nodes.length + results[category].edges.length} results</p>
                  </div>
                  <div className="h-[1px] w-full bg-offwhite-300"></div>
                  {[].concat(...Object.values(results[category])).map((item, index) => (
                    <div
                      key={index}
                      className="flex flex-col hover:bg-slate-300 p-2 cursor-pointer"
                      title={JSON.stringify(item)}
                      onClick={() => {
                        CATEGORY_ACTIONS[category](
                          {
                            nodes: results[category].nodes.includes(item) ? [item] : [],
                            edges: results[category].edges.includes(item) ? [item] : [],
                          },
                          dispatch,
                        );
                      }}
                    >
                      <p style={{ fontSize: '14px', fontWeight: '600' }}>
                        {item.key.slice(0, 18) || item.id.slice(0, 18) || Object.values(item)[0].slice(0, 18)}
                      </p>
                      <p className="font-thin text-slate-400" style={{ fontSize: '12px' }}>
                        {JSON.stringify(item).substring(0, 40)}...
                      </p>
                    </div>
                  ))}
                </div>
              );
            }
          })}
        </div>
      )}
    </div>
  );
}
